#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h> 
#include <errno.h>
#include <sys/time.h>

#include "config.h"
#include "engine.h"
#include "portaloo.h"

int host_write(int fd, char *ptr)
{
	int err=write(fd, ptr, strlen(ptr));
	return err;
}

char host_rbuf[16384];
int host_rptr;
int host_rlen;

static int host_getblk(int fd)
{
	int rlen;
	fd_set rfd;
	struct timeval tv;
	
	while(1)
	{
		rlen=read(fd, host_rbuf, sizeof(host_rbuf));
		if(rlen>0)
		{
			host_rlen=rlen;
			host_rptr=0;
			return host_rlen;
		}
		if(rlen==0)
		{
			return 0;
		}
		if(rlen==-1 && errno!=EWOULDBLOCK)
		{
			return 0;
		}
		tv.tv_sec=180;
		tv.tv_usec=0;
		FD_ZERO(&rfd);
		FD_SET(fd, &rfd);
		
		if(select(fd+1, &rfd, NULL, NULL, &tv)<1)
			return 0;
	}
}

char *host_readline(int fd)
{
	static char buf[16384];
	char *bp=buf;
	
	while(bp-buf<16383)
	{
		if(host_rptr==host_rlen)
			if(host_getblk(fd)==0)
			{
				if(bp==buf)
					return NULL;
				else
					*bp=0;
				return buf;
			}
		*bp=host_rbuf[host_rptr++];
		if(*bp=='\n')
		{
			*bp=0;
//			printf(">%s\n",buf);
			return buf;
		}
		if(*bp!='\r')
			bp++;
	}
	*bp=0;
	return buf;
}

int host_readchar(int fd)
{
	if(host_rptr==host_rlen)
		if(host_getblk(fd)==0)	
			return EOF;
			
	return host_rbuf[host_rptr++];
}

	

static int host_connect_attempt(struct in_addr ia, int port)
{
	int s=socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	
	struct sockaddr_in sin;
	
	fd_set wfd;
	struct timeval tv;
	
	if(s==-1)
	{
		perror("socket");
		return -1;
	}
	
	if(fcntl(s, F_SETFL, FNDELAY)==-1)
	{
		perror("nonblocking");
		close(s);
		return -1;
	}

	sin.sin_family = AF_INET;	
	sin.sin_addr   = ia;
	sin.sin_port   = htons(port);
	
	if(connect(s, (struct sockaddr *)&sin, sizeof(sin))==-1 && errno != EINPROGRESS)
	{
		perror("connect");
		close(s);
		return -1;
	}	
	
	tv.tv_sec = 90;		/* We use 90 second timeouts for now */
	tv.tv_usec = 0;
	
	FD_ZERO(&wfd);
	FD_SET(s, &wfd);
	
	switch(select(s+1, NULL, &wfd, NULL, &tv))
	{
		case 0:
			/* Time out */
			close(s);
			return -1;
		case -1:
			/* Ermm.. ?? */
			perror("select");
			close(s);
			return -1;
	}
	
	return s;
}

#ifndef PROXY_HOST
 
int host_connect(const char *host, int port)
{
	struct hostent *h;
	int i;
	int s;
	
	h=gethostbyname(host);
	if(h==NULL)
	{
		fprintf(stderr,"unable to resolve '%s'.\n", host);
		return -1;
	}
	
	
	for(i=0; h->h_addr_list[i]; i++)
	{
		struct in_addr ia;
		memcpy(&ia, h->h_addr_list[i],4);
		s=host_connect_attempt(ia, port);
		if(s != -1)
			return s;
	}
	fprintf(stderr, "unable to connect to '%s'.\n", host);
	return -1;
}

int host_get(int s, char *host, char *path)
{
	char buf[1024];
	snprintf(buf, 1024, "GET %s HTTP/1.0\r\nhost: %s\r\n\r\n",
		path, host);
//	printf("%s\n", buf);
	host_write(s, buf);
}

#else

 
int host_connect(const char *host, int port)
{
	struct hostent *h;
	int i;
	int s;
	
	h=gethostbyname(PROXY_HOST);
	if(h==NULL)
	{
		fprintf(stderr,"unable to resolve '%s'.\n", host);
		return -1;
	}
	
	
	for(i=0; h->h_addr_list[i]; i++)
	{
		struct in_addr ia;
		memcpy(&ia, h->h_addr_list[i],4);
		s=host_connect_attempt(ia, PROXY_PORT);
		if(s != -1)
			return s;
	}
	fprintf(stderr, "unable to connect to '%s'.\n", PROXY_HOST);
	return -1;
}

int host_get(int s, char *host, char *path)
{
	char buf[1024];
	snprintf(buf, "GET http://%s%s HTTP/1.0\r\nhost: %s\r\n\r\n",
		path, host);
	host_write(s,buf);
}



#endif
