/*
 * Channel retrieval tool
 *
 * (C) 1998 Alan Cox.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h> 
#include <errno.h>
#include <sys/time.h>

#include <glib.h>
#include <gnome-xml/parser.h>

#include "config.h"
#include "engine.h"
#include "portaloo.h"

int xmlSubstituteEntitiesDefaultValue=1;	/* DV thinks of everything */

static int wipe_trackers=0;

FILE *output;


static char *layer_find(xmlNodePtr node, char *match, char *fail)
{
	while(node!=NULL)
	{
//		xmlDebugDumpNode(stdout, node, 32);
//		printf("%s.\n", node->name);
		if(strcasecmp(node->name, match)==0)
		{
			if(node->childs != NULL && node->childs->content != NULL)
				return node->childs->content;
			else
				return fail;
		}
		node = node->next;
	}
	return fail;
}

static char *layer_find_url(xmlNodePtr node, char *match, char *fail)
{
	char *p=layer_find(node, match, fail);
	char *r=p;
	static char *wb=NULL;
	char *w;
	
	if(wb)
		free(wb);
		
	wb=w=malloc(3*strlen(p));
	
	if(w==NULL)
	{
		fprintf(stderr, "Out of memory\n");
		exit(1);
	}
	
	if(*r==' ') r++;	/* Fix UF bug */

	while(*r)
	{
		if(memcmp(r, "&amp;", 5)==0)
		{
			*w++='&';
			r+=5;
			continue;
		}
		if(memcmp(r, "&lt;", 4)==0)
		{
			*w++='<';
			r+=4;
			continue;
		}
		if(memcmp(r, "&gt;", 4)==0)
		{
			*w++='>';
			r+=4;
			continue;
		}
		if(*r=='"'||*r==' ')
		{
			*w++='%';
			*w++="0123456789ABCDEF"[*r/16];
			*w++="0123456789ABCDEF"[*r&15];
			r++;
			continue;
		}
		*w++=*r++;
	}
	*w=0;
	return wb;
}

static void tree_walk(xmlNodePtr root)
{
	xmlNodePtr walk;
	xmlNodePtr rewalk = root;
	xmlNodePtr channel = NULL;
	xmlNodePtr image = NULL;
	xmlNodePtr item[16];
	int items=0;
	int limit = 10;
	int i;
	char *t;
	char n[512];
	
	do
	{
		walk=rewalk;
		rewalk=NULL;
	
		while(walk!=NULL)
		{
//			printf("%p, %s\n", walk, walk->name);
			if(strcasecmp(walk->name, "rdf")==0)
			{
				rewalk=walk->childs;
				walk=walk->next;
				continue;
			}
			if(strcasecmp(walk->name, "rss")==0)
			{
				rewalk=walk->childs;
				walk=walk->next;
				continue;
			}
			/* This is the channel top level */
//			printf("Top level '%s'.\n", walk->name);
			if(strcasecmp(walk->name, "channel")==0)
			{
				channel = walk;
				rewalk = channel->childs;
			}
			if(strcasecmp(walk->name, "image")==0)
				image = walk;
			if(strcasecmp(walk->name, "item")==0 && items < 16)
				item[items++]=walk;
			walk=walk->next;
		}
	
	}
	while(rewalk);
	
	if(channel==NULL)
	{
		fprintf(stderr, "No channel definition.\n");
		exit(1);
	}
	fprintf(output, "%s\n", 
		t=layer_find(channel->childs, "title", "No title"));
	fprintf(output, "%s\n", 
		layer_find(channel->childs, "link", ""));
	
	fprintf(output, "%s\n--\n",
		layer_find(channel->childs, "description", ""));
	if(image && !wipe_trackers)
	{		
		fprintf(output, "<A href=\"%s\">",
			layer_find_url(image->childs, "link", ""));
		fprintf(output,"<IMG src=\"%s\" alt=\"%s\" align=right width=96 height=48></A>",
			layer_find_url(image->childs, "url", ""),
			layer_find(image->childs, "title", ""));
		limit=8;
	}

	fprintf(output, "<br clear=all><FONT size=\"-1\" face=\"helvetica\"><P><UL>\n");		
	for(i=0;i<items ;i++)
	{
		char *p= layer_find(item[i]->childs, "title", "No information");
		
		if(i==limit)
			fprintf(output, "--\n");
		if(wipe_trackers)
		{
			char *p=layer_find_url(item[i]->childs, "link", "");
			char *x=strchr(p,'?');
			unsigned char *r, *w;
			int n;
			if(x==NULL)
				continue;
			x++;
			r=x;
			w=x;
			while(*r)
			{
				if(*r=='+')
					*w++=' ';
				else if(*r=='%')
				{
					sscanf(r+1, "%02x", &n);
					*w++=n;
					r+=2;
				}
				else *w++=*r;
				r++;
			}
			*w=0;
			fprintf(output, "<LI><A href=\"%s\">\n", x+4);
		}
		else
			fprintf(output, "<LI><A href=\"%s\">\n", layer_find_url(item[i]->childs, "link", ""));
		fprintf(output, "%s\n", p);
		fprintf(output, "</A>\n");
	}
	fprintf(output, "</UL></FONT>\n");
}

static char document[65536];

int main(int argc, char *argv[])
{
	int s;
	int head=1;
	int docp=0;
	xmlDocPtr doc;
	char buf[512];
	char nam[512];
	char *prog = argv[0];
	int port = 80;
	
	char *p;
	
	/*
	 *	The jerks at webstartshere add tracking stuff to their
	 *	rdf of sites. Strike one for privacy and wipe it back
	 *	out again.
	 */
	 
	if(argv[1] && strcmp(argv[1], "-w")==0)
	{
		wipe_trackers=1;
		argv++;
		argc--;
	}
		
	if(argv[1] && strcmp(argv[1], "-p")==0)
	{
		if(argv[2]==NULL)
		{
			fprintf(stderr, "rdf-engine: A port number is required.\n");
			exit(1);
		}
		port=atoi(argv[2]);
		argv+=2;
		argc-=2;
	}
		
	if(argc!=5)
	{
		fprintf(stderr, "%s: chan ip host path.\n", prog);
		exit(1);
	}
	
	
	snprintf(buf, sizeof(buf), "%s/%s.new", PORTALOO_BASE, argv[1]);
	snprintf(nam, sizeof(buf), "%s/%s.out", PORTALOO_BASE, argv[1]);
	
	output=fopen(buf, "w");
	if(output==NULL)
	{
		perror(buf);
		return 1;
	}
	
	s=host_connect(argv[2],port);
	if(s==-1)
		return 1;
	
	host_get(s, argv[3], argv[4]);
			
	while((p=host_readline(s))!=NULL)
	{
		if(!head && docp+strlen(p) < 65535)
		{
			memcpy(document+docp, p, strlen(p));
			docp+=strlen(p);
			document[docp++]='\n';
		}
		if(head && *p == 0)
			head = 0;
	}
	close(s);
	document[docp]=0;
	
	doc=xmlParseMemory(document, docp);
	
	if(doc==NULL)
	{
		fprintf(stderr, "Unable to parse document.\n");
		exit(1);
	}
	
	tree_walk(doc->root);
	
	if(rename(buf, nam))
		perror("rename");
	return 0;
}
	