/*
 * Cabletel TV parser tool for Portaloo
 *
 * (C) 1999 Alan Cox.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <fcntl.h> 
#include <errno.h>
#include <sys/time.h>

#include "config.h"
#include "portaloo.h"

static FILE *datafile;
static char host_rbuf[4096];
static int host_rptr;
static int host_rlen;

static int host_getblk(FILE *f)
{
	int rlen;
	fd_set rfd;
	struct timeval tv;
	
	while(1)
	{
		rlen=fread(host_rbuf, 1, sizeof(host_rbuf), f);
		if(rlen>0)
		{
			host_rlen=rlen;
			host_rptr=0;
			return host_rlen;
		}
		if(rlen==0)
		{
			return 0;
		}
		if(rlen==-1 && errno!=EWOULDBLOCK)
		{
			return 0;
		}
	}
}

char *host_readline(FILE *f)
{
	static char buf[4096];
	char *bp=buf;
	
	while(bp-buf<4095)
	{
		if(host_rptr==host_rlen)
			if(host_getblk(f)==0)
				return NULL;
		*bp=host_rbuf[host_rptr++];
		if(*bp=='\n')
		{
			*bp=0;
			while(bp>buf)
			{
				if(bp[-1]==' ')
					*--bp=0;
				else
					break;
			}
			printf(">'%s'\n",buf);
			return buf;
		}
		if(*bp!='\r')
			bp++;
	}
}

FILE *host_connect(int page)
{
	char buf[256];
	snprintf(buf, 256, "./vtxget %d.0", page);
	datafile = popen(buf, "r");
	return datafile;
}

int host_close(FILE *f)
{
	pclose(f);
}

struct line
{
	struct line *next;
	char *data;
	int page;
};

struct line *lines;

static void add_text(int sp, char *line)
{
	struct line *n=(struct line *)malloc(sizeof(*n));
	struct line **l=&lines;
	if(n==NULL || (n->data=strdup(line))==NULL)
	{
		fprintf(stderr, "Out of memory.\n");
		exit(1);
	}
	n->page=sp;
	while(*l!=NULL)
	{
		if((*l)->page > sp)
			break;
		l=&((*l)->next);
	}
	n->next = *l;
	*l=n;
}

static char *make_title(char *p, char *ep)
{
	static char format[256];
	char *x=format;
	
//	char *prem;
	
	ep--;
	while(ep>p && isspace(*ep))
		ep--;

#if 0		
	prem=strrchr(p, '(');
	if(prem && strcasecmp(prem, "(premier)", 9)==0)
		*prem=0;
#endif		
	while(*p && p<=ep)
	{
		if(*p==' ')
		{
			strcpy(x,"%20");
			x+=3;
		}
		else if(*p=='&')
		{
			sprintf(x,"%%%02x",'&');
		}
		else
			*x++=*p;
		p++;
	}
	*x=0;
	return format;
}

static char *film_url(char *x)
{
	static char format[1024];
	char *y=x+strlen(x)-1;
	if(strlen(x)<6)
		return NULL;
		
	/* Look for (year) - skip (premier) that Sky uses */
	
	y++;
	do
	{
		y--;
		while(y>x+1 && *y!='(')
			y--;
	}		
	while(strncasecmp(y, "(premier)", 9)==0);
		
	if(*y!='(' || y[5]!=')' || ( y[1]!='1' && y[1]!='2'))
		return NULL;
	if(strncasecmp(x,"Film:",5)==0)
	{
		x+=5;
		if(*x==' ')
			x++;
	}
	snprintf(format,1024,"http://us.imdb.com/M/title-substring?title=%s&tv=off",
		make_title(x,y));
	return format;
}
	
static char *html_quote(char *x)
{
	char *p;
	static char *pb;

	if(pb!=NULL)
	{
		free(pb);
		pb=NULL;
	}
	
	p=malloc(strlen(x)*5+1);
	pb=p;
	if(p==NULL)
		return x;
	while(*x)
	{
		if(*x=='<')
		{
			strcpy(p,"&lt;");
			p+=4;
		}
		else if(*x=='>')
		{
			strcpy(p,"&gt;");
			p+=4;
		}
		else if(*x=='&')
		{
			strcpy(p,"&amp;");
			p+=5;
		}
		else
			*p++=*x;
		x++;
	}
	*p=0;
	return pb;
}


static char *add_string(char *old, char *new)
{
	char *p=malloc(strlen(old)+strlen(new)+2);
	if(p==NULL)
	{
		fprintf(stderr,"Out of memory.\n");
		exit(1);
	}
	sprintf(p, "%s %s", old, new);
	return p;
}

int main(int argc, char *argv[])
{
	FILE *s;
	int pnum;
	int ct=0;	
	char *p;
	char buf[256];
	char base[256];
	FILE *f;
	int subpage = 0;	/* Teletext sub page identity */
	int titled = 0;		/* Channel name found */
	int mode =0;		/* State in page parse */
	int hanging = 0;	/* Hanging </A> due */
	struct line *l;
	
	if(argc!=3)
	{
		fprintf(stderr,"%s [num] [page]\n", argv[0]);
		exit(1);
	}	
	
	if(sscanf(argv[2], "%d", &pnum)!=1)
	{
		fprintf(stderr, "%s: invalid page.\n", argv[2]);
		exit(1);
	}
	
	snprintf(buf, sizeof(buf), PORTALOO_BASE"%s.new", argv[1]);
	snprintf(base, sizeof(buf), PORTALOO_BASE"%s.out", argv[1]);
	
	f=fopen(buf, "w");
	if(f==NULL)
	{
		perror(buf);
		exit(1);
	}
	
	s=host_connect(pnum);
	if(s==NULL)
		return 1;
	
	
	while((p=host_readline(s))!=NULL)
	{
		char *app;
rescan:
		if(*p=='{')
		{
			/* Begin page */
			sscanf(p, "{PAGE %*d/%d}", &subpage);
			/* Discard channel guide line */
			p=host_readline(s);
			if(strncmp(p+9, "THE CHANNEL GUIDE", 17))
			{
				fprintf(stderr, "Wrong channel ?\n");
				exit(1);
			}
			/* blank, blank */
			host_readline(s);
			host_readline(s);
			/* channel name */
			p=host_readline(s);
			if(p==NULL)
				break;
			/* Ok we have the name */
			if(!titled)
			{
				fprintf(f,"%s\n\n[%d] %s\n--\n", p+4, pnum,  p+4);
				fprintf(f,"<br clear=all><FONT size=\"-1\" face=\"helvetica\"><P><UL>");
				titled=1;
			}
			mode = 1;
			continue;
		}
		if(mode==0)
			continue;
		/* In the body */
		if(mode==1)
		{
			if(strncasecmp(p+4, "Ch:", 3)==0)
			{
				mode++;
				continue;
			}
			continue;
		}
		if(strlen(p)<5)
			continue;
		/* Continuation, end marker or data ? */
		if(isdigit(p[4]))
		{
			char *f;
			char *x;
			if(hanging)
			{
				add_text(subpage, "\n</A>\n<LI>");
				hanging=0;
			}
			else
				add_text(subpage, "\n<LI>");
				
			x=strdup(p);
			if(x==NULL)
			{
				fprintf(stderr, "Out of memory.\n");
				exit(1);
			}
			do
			{
				p=host_readline(s);
				if(p[4]==' ')
					x=add_string(x, p+9);
			}
			while(p[4]==' ');
			
			f=film_url(x+4);
			if(f)
			{
				add_text(subpage, "<A href=\"");
				add_text(subpage, f);
				add_text(subpage, "\">");
				hanging=1;
			}
			add_text(subpage, html_quote(x+4));
			free(x);
			goto rescan;
		}
		else if(p[4]==' ')
		{
 			add_text(subpage, html_quote(p+9));
 		}
		else	/* End break */
			continue;
	}
	if(hanging)
		add_text(subpage, "\n</A>");
	for(l=lines;l!=NULL;l=l->next)
	{
		if(strchr(l->data, '\n'))
		{
			ct++;
			if(ct==10)
				fprintf(f, "\n--");
		}
		fprintf(f, "%s", l->data);
	}
		
	fprintf(f,"\n</UL></FONT>\n");
	fclose(f);
	host_close(s);
	/*
	 *	Dead data ?
	 */
	if(!titled)
		exit(1);
	if(rename(buf,base)==-1)
	{
		perror("rename");
		exit(1);
	}
	exit(0);
}
	