#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <time.h>

#include "config.h"
#include "md5.h"
#include "portaloo.h"

static u8 secret[8];
struct portaloo self;	/* Portaloo prefs */

int avoid_cache;

int load_secret(void)
{
	int fd=open(PORTALOO_SECRET, O_RDONLY);
	if(fd==-1)
		return -1;
	if(read(fd, secret, 8)!=8)
		return -1;
	close(fd);
	return 0;
}

void init_portaloo(struct portaloo *p, char *name)
{
	memset(p, 0, sizeof(*p));
	strncpy(p->uname, name, 32);
	p->magic = PORTALOO_MAGIC;
	p->version = PORTALOO_VERSION;
	p->subscribed[0] = 1;
	p->subscribed[1] = 2;
	p->subscribed[2] = 3;
	p->subscribed[3] = 4;
	p->subscribed[4] = 5;
	p->subscribed[5] = 6;
}

int check_portaloo(struct portaloo *p)
{
	char *n;
	struct portaloo tp;
	struct portaloo_v1 *op=(struct portaloo_v1 *)p;
	
	if(p->magic != PORTALOO_MAGIC)
		return -1;
		
	if(p->version == 1)
	{
		memset(&tp, 0, sizeof(tp));
		tp.magic=PORTALOO_MAGIC;
		tp.version=2;
		memcpy(tp.subscribed, op->subscribed, 64);
		memcpy(tp.flags, op->flags, 32);
		memcpy(tp.uname, op->uname, 32);
		tp.laston = op->laston;
		memcpy(tp.count, op->count, 4);
		tp.flags[0]&=FLAGS0_SHOW_TEXT|FLAGS0_SHOW_LONG;
		memcpy(p, &tp, sizeof(tp));
	}
	if(p->version != PORTALOO_VERSION)
		return -1;
	
	n=p->uname;
	while(*n)
	{
		if(*n<32)
			return -1;
		n++;
	}
	/* Name requires trust */
	if((p->flags[0]&FLAGS0_NAME_SET) && !(p->flags[0]&FLAGS0_MD5_HASH))
	{
		p->flags[0]&=~FLAGS0_NAME_SET;
	}
	if(p->flags[0]&FLAGS0_MD5_HASH)
	{
		struct portaloo tp=*p;
		struct md5_ctx ctx;
		md5_init_ctx(&ctx);
		tp.flags[8]=0;
		tp.flags[9]=0;
		tp.flags[10]=0;
		tp.flags[11]=0;
		tp.flags[12]=0;
		tp.flags[13]=0;
		tp.flags[14]=0;
		tp.flags[15]=0;
		md5_process_bytes(secret, sizeof(secret), &ctx);
		md5_process_bytes(&tp, sizeof(tp), &ctx);
		md5_finish_ctx(&ctx, &tp.flags[8]);	
		if(memcmp(&tp.flags[8], &p->flags[8],16)!=0)
			tp.flags[0]&=~FLAGS0_NAME_SET; 
	}
	return 0;
}

void issue_portaloo_cookie(struct portaloo *p)
{
	struct portaloo tp = *p;
	struct tm *t;
	u8 *pbytes = (u8 *)&tp;
	int n;
	int zbytes=0;
	time_t now;
	struct md5_ctx ctx;

	time(&now);
	t=gmtime(&now);
	tp.laston=now;

	tp.flags[8]=0;
	tp.flags[9]=0;
	tp.flags[10]=0;
	tp.flags[11]=0;
	tp.flags[12]=0;
	tp.flags[13]=0;
	tp.flags[14]=0;
	tp.flags[15]=0;
	tp.flags[0]|=FLAGS0_MD5_HASH;
	
	md5_init_ctx(&ctx);
	md5_process_bytes(secret, sizeof(secret), &ctx);
	md5_process_bytes(&tp, sizeof(tp), &ctx);
	md5_finish_ctx(&ctx, &tp.flags[8]);	

	if(avoid_cache==0)
	{
		printf("Cache-control: private, max-age=3600\n");
	}
	else
	{	
		printf("Pragma: no-cache\n");
		printf("Cache-control: no-cache\n");
	}
	printf("Set-cookie: Context=");
	for(n=0;n<sizeof(*p);n++)
	{
		if(pbytes[n]==0)
			zbytes++;
		else
		{
			if(zbytes>1)
				printf("Z%02X", zbytes);
			if(zbytes==1)
				printf("00");
			zbytes=0;
			printf("%02X",pbytes[n]);
		}
	}
	if(zbytes>0)
		printf("Z%02X", zbytes);
	printf("; expires=Thu, 01 Jan %04d 12:00:00; path=/; domain="PORTALOO_COOKIE"\n",
		t->tm_year+1902);
}


int decode_portaloo_cookie(struct portaloo *p, char *n)
{
	int v;
	char *name;
	char *good=NULL;
	u8 *pbytes = (u8 *)p;

	while(n!=NULL)
	{
		name=n;
		n=strchr(n,'=');
		if(n==NULL)
			return 0;
		*n++=0;
		if(strcmp(name, "Context")==0)
		{
			good=n;
		}
		n=strchr(n,';');
		if(n==NULL)
			break;
		do
		{
			n++;
		}
		while(*n && isspace(*n));
	}

	if(good==0)
		return 0;
		
	/*
	 *	Now get the last cookie instance. This works around the
	 *	bugs in netscrap 4.
	 */
	 
	n=good;
	
	for(v=0;v<sizeof(*p);v++)
	{
		u8 buf[3];
		int x;
		int type=1;
		if(n[0]=='Z')
		{
			type=2;
			n++;
		}
		if(n[0]<'0'|| n[0]>'F')
			return -1;
		if(n[1]<'0'|| n[1]>'F')
			return -1;
		memcpy(buf, n, 2);
		n+=2;
		buf[2]=0;
		if(sscanf(buf, "%X", &x)<1)
		{
			return -1;
		}
		if(type==1)
			*pbytes++=x;
		else if(type==2)
		{
			if(v+x > sizeof(*p))
			{
				return -1;
			}
			memset(pbytes, 0, x);
			pbytes+=x;
			v+=x-1;
		}
	}
	if(check_portaloo(p))
		return 0;
	return 1;
}

