#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdbm.h>
#include <time.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/wait.h>

#include "config.h"
#include "struct.h"
#include "proto.h"
#include "portaloo.h"

extern FILE *debug;

int user_valid(char *username, char *password)
{
	GDBM_FILE db;
	datum key, data;
	char path[512];

	if (username==NULL || password==NULL) return(FALSE);

	key.dptr=strdup(username);
	key.dsize=strlen(username)+1;

	snprintf(path, 512, "%s/user.db", HOMEPATH);
	if ((db=gdbm_open(path, 1024, GDBM_READER, 0600, NULL))==NULL)
	{
		printf("Opening: %s\n", gdbm_strerror(gdbm_errno));
		exit(0);
	}

	data=gdbm_fetch(db, key);
	gdbm_close(db);

	self.flags[0]&=~FLAGS0_NAME_SET;
	if (data.dptr==NULL) /* not found */
		return(FALSE);

/*	fprintf(stderr,"user.db query u='%s' p='%s' [%d]  give='%s'\n",username,data.dptr,data.dsize, password); */
	if (!strcmp(password, data.dptr)) /* match */
	{
		self.flags[0]|=FLAGS0_NAME_SET;
		return(TRUE);
	}
	
	return(FALSE);
}

static char *make_password(void)
{
	static char buf[8];
	static char *randstr="ABCDEFHIJKLMNOP0123456789QRSTUVWXYZabcdefgh!^klmnopqrstuv$wxyz@";
	long t=time(NULL);
	int i;
	
	srand(t+getpid());
	for(i=0;i<8;i++)
		buf[i]=randstr[(rand()>>3)%strlen(randstr)];
	buf[8]=0;
	return buf;
}
	
int user_add(char *name, char *email)
{
	GDBM_FILE db;
	datum key, data;
	char path[512];
	char *pass = make_password();
	FILE *p;
	int pipes[2];
	
	key.dptr=name;
	key.dsize=strlen(name)+1;

	data.dptr=pass;
	data.dsize=strlen(pass)+1;

	snprintf(path, 512, "%s/user.db", HOMEPATH);
	if ((db=gdbm_open(path, 1024, GDBM_WRCREAT, 0600, NULL))==NULL)
	{
		printf("\n\nOpening: %s\n", gdbm_strerror(gdbm_errno));
		exit(0);
	}

	if (gdbm_store(db, key, data, GDBM_INSERT) !=0)
	{	
		gdbm_close(db);
		return -1;
	}
	gdbm_close(db);

	key.dptr=name;
	key.dsize=strlen(name)+1;

	data.dptr=email;
	data.dsize=strlen(email)+1;

	snprintf(path, 512, "%s/email.db", HOMEPATH);
	if ((db=gdbm_open(path, 1024, GDBM_WRCREAT, 0600, NULL))==NULL)
	{
		printf("\n\nOpening: %s\n", gdbm_strerror(gdbm_errno));
		exit(0);
	}

	if (gdbm_store(db, key, data, GDBM_REPLACE) !=0)
	{	
		gdbm_close(db);
		return -1;
	}
	gdbm_close(db);
	
	pipe(pipes);
	
	switch(fork())
	{
		case 0:
			dup2(pipes[0], 0);
			close(pipes[1]);
			execlp("/usr/bin/mail", "mail", "-s", "Portaloo Password", email, NULL);
			execlp("/bin/mail", "mail", email, NULL);
			exit(1);		
		case -1:printf("\n\nFork failed!\n");
			exit(1);
		default:	
			close(pipes[0]);
			p=fdopen(pipes[1], "w");
			fprintf(p, "Your username is: %s\n", name);
			fprintf(p, "Your portaloo password is: %s\n\n", pass);
			fprintf(p, "--\nYour friendly portaloo: "PORTALOO_COOKIE"\n");
			fclose(p);
			wait(NULL);
	}
	return 0;
}

int user_lost(char *name)
{
	GDBM_FILE db;
	datum key, data;
	char path[512];
	char *pass;
	FILE *p;
	int pipes[2];
	char *email;
	
	key.dptr=name;
	key.dsize=strlen(name)+1;

	snprintf(path, 512, "%s/email.db", HOMEPATH);
	if ((db=gdbm_open(path, 1024, GDBM_READER, 0600, NULL))==NULL)
	{
		printf("\n\nOpening: %s\n", gdbm_strerror(gdbm_errno));
		exit(0);
	}

	data=gdbm_fetch(db, key);
	gdbm_close(db);
	
	if(data.dptr==NULL)
		return -1;

	email=strdup(data.dptr);

	key.dptr=name;
	key.dsize=strlen(name)+1;

	snprintf(path, 512, "%s/user.db", HOMEPATH);
	if ((db=gdbm_open(path, 1024, GDBM_READER, 0600, NULL))==NULL)
	{
		printf("\n\nOpening: %s\n", gdbm_strerror(gdbm_errno));
		exit(0);
	}

	data=gdbm_fetch(db, key);
	gdbm_close(db);
	
	if(data.dptr==NULL)
		return -1;
	
	pass=data.dptr;
	
	pipe(pipes);
	
	switch(fork())
	{
		case 0:
			dup2(pipes[0], 0);
			close(pipes[1]);
			execlp("/usr/bin/mail", "mail", "-s", "Portaloo Password", email, NULL);
			execlp("/bin/mail", "mail", email, NULL);
			exit(1);		
		case -1:printf("\n\nFork failed!\n");
			exit(1);
		default:	
			close(pipes[0]);
			p=fdopen(pipes[1], "w");
			fprintf(p, "Your username is: %s\n", name);
			fprintf(p, "Your portaloo password is: %s\n\n", pass);
			fprintf(p, "--\nYour friendly portaloo: "PORTALOO_COOKIE"\n");
			fclose(p);
			wait(NULL);
	}
	return 0;
}


void print_login(int size)
{
	if ((self.flags[0]&FLAGS0_NAME_SET)!=0)
	{
		printf("<table border=0 cellpadding=3>\n");
		printf("<tr><td><font face=\"helvetica\">You are logged in as <b>%s</b></font></td></tr>\n",self.uname);
		printf("<tr><td><A href=\"%s?cookie_user=logout\">Logout</A></td></tr>\n", 
			CGIPATH);
		printf("</table>\n");

	}else
	{
		printf("<form action=%s method=get>\n", CGIPATH);
		printf("<input type=hidden name=cookie_user value=login>\n");
		printf("<table border=0 width=99%%>\n");
		printf("<tr><td bgcolor=\"#000000\" colspan=2><font face=\"helvetica\" color=\"white\" size=%s%d>\n", size>=-1?"+":"", size+1);
		printf("<b>Login</b></td></tr>\n");
		printf("<tr><td>");
		printf("<font face=helvetica size=%s%d>\n", size>=0?"+":"", size);
		printf("Name:</td><td><input type=text name=username size=8 maxsize=16></TD>\n");
		printf("</tr><tr><td>");
		printf("<font face=helvetica size=%s%d>\n", size>=0?"+":"", size);
		printf("Pass:</TD><TD><input type=password name=password size=8 maxsize=8></td></tr>\n");
		printf("<tr><td colspan=2><input type=submit name=submit value=Login><br>\n");
		printf("<A href=\"%s?new_user=form\">No account ?</A><br>\n", CGIPATH);
		printf("</font>\n");
		printf("</td></tr>\n");
		printf("</table>\n");
		printf("</form>\n");
	}
}


char *user_detail(char *username, char *field)
{
	GDBM_FILE db;
	datum key, data;
	char path[512];

	if (username==NULL || field==NULL) return(FALSE);

	snprintf(path, 512, "%s:%s", username, field);
	key.dptr=strdup(path);
	key.dsize=strlen(path)+1;

	snprintf(path, 512, "%s/user.db", HOMEPATH);
	if ((db=gdbm_open(path, 1024, GDBM_READER, 0600, NULL))==NULL)
	{
		printf("Opening: %s\n", gdbm_strerror(gdbm_errno));
		exit(0);
	}

	data=gdbm_fetch(db, key);
	gdbm_close(db);

	if (data.dptr==NULL) /* not found */
		return(NULL);
	else
		return(data.dptr);
}
